<template>
  <div class="corporate-customers">
    <div class="full-page bg" v-show="componentName === 'table'">
      <div class="search-box">
        <SuperSearch :searchForm="queryParams" :basisOption="basisOption" @is-show-advance="advanced()"
                     @search="handleSearch" @reset="loadData" />
      </div>
      <div class="main-table-container">
        <div class="table-container" style="padding: 20px">
          <div style="margin-bottom: 10px" ref="btnPanel">
            <el-button
              icon="el-icon-s-claim"
              type="warning"
              plain
              class="btn"
              :disabled="!this.currentDataDel.length"
              @click="markAsRead()"
            >
              标为已读
            </el-button>
            <el-button
              icon="el-icon-delete"
              type="danger"
              plain
              class="btn btn-delete"
              :disabled="!this.currentDataDel.length"
              @click="handleDelete()"
            >
              删除
            </el-button>
          </div>
          <div class="table-content" ref="tabelRef">
            <el-table stripe v-loading="tableLoading" :data="tableData" :row-key="({id}) => id"
                      :max-height="tableMaxHeight" class="table-list" border @selection-change="handleSelectionChange"
                      @sort-change="sortChange">
              <el-table-column type="selection" width="55" align="center"></el-table-column>
              <el-table-column type="index" width="70" label="序号" align="center">
              </el-table-column>
              <el-table-column
                prop="subject"
                sortable
                show-overflow-tooltip
                min-width="180"
                align="center"
                label="标题"
              />
              <el-table-column
                prop="receiveTime"
                show-overflow-tooltip
                min-width="120"
                align="center"
                label="消息状态"
              >
                <template slot-scope="{row:{receiveTime}}">
                  <el-tag v-if="receiveTime" type="warning" size="medium">已读</el-tag>
                  <el-tag v-else type="danger">未读</el-tag>
                </template>
              </el-table-column>
              <el-table-column
                prop="owner"
                show-overflow-tooltip
                sortable
                min-width="120"
                align="center"
                label="发件人">
              </el-table-column>
              <el-table-column
                v-if="tableData.some( i => i.receiveTime)"
                prop="receiveTime"
                show-overflow-tooltip
                align="center"
                label="已读时间"
                min-width="90"
              />
              <el-table-column
                prop="createTime"
                show-overflow-tooltip
                align="center"
                label="发信时间"
                :min-width="tableData.some( i => i.receiveTime) ? '90' : '180'"
              />
              <el-table-column show-overflow-tooltip min-width="120" align="center" fixed="right" label="操作">
                <template v-slot="{ row }">
                  <el-link v-if="!row.receiveTime"
                           type="warning"
                           :underline="false"
                           style="margin-right: 15px;"
                           @click="markAsRead(row)">
                    标为已读
                  </el-link>
                  <el-link type="primary"
                           :underline="false"
                           style="margin-right: 15px;"
                           @click="showDialog('isShowDetail',row)">
                    详情
                  </el-link>
                  <el-link type="danger" :underline="false" style="margin-right: 15px;" @click="handleDelete(row)">
                    删除
                  </el-link>
                </template>
              </el-table-column>
            </el-table>
            <pagination ref="paginationPanel" style="margin-top: 10px" :pageResult="queryParams"
                        @size-change="handleSizeChange" @current-change="handleCurrentChange" />
          </div>
        </div>
      </div>
    </div>

    <!--  详情弹层  -->
    <detail-dialog
      :isShowDetail.sync="isShowDetail"
      width="40%"
      title="消息通知"
      :rowInfo="rowInfo"
      @close-dialog="closeDetail"
    />

  </div>
</template>

<script>
import pagination from '@/components/pagination'
import {TableMixin} from '@/mixins/tableMixin'
import detailDialog from './components/DetailDialog.vue'
import SuperSearch from '@/components/SuperSearch/index.vue'
import {messageReceiver, removeMessage, markMessage} from '@/api/MessageReminder'
import SuperDialog from '@/components/SuperDialog/index.vue'
import DetailDialog from '@/views/MessageReminder/components/DetailDialog.vue'


export default {
  name: 'MessageReminder',
  components: {
    DetailDialog,
    SuperDialog,
    SuperSearch,
    pagination
  },
  mixins: [TableMixin],
  data() {
    return {
      isShowDetail: false,
      tableMaxHeight: '600px',
      loading: false,
      tableLoading: false,
      componentName: 'table',
      tableData: [],
      queryParams: {
        page: 1,
        pageSize: 20,
        total: 0,
        subject: '',
        receiveTime: '1'
      },
      currentDataDel: [],
      sorter: [
        {
          direction: 'ASC',
          property: 'createTime'
        },
        {
          direction: '',
          property: 'subject'
        },
        {
          direction: '',
          property: 'owner'
        }
      ],
      basisOption: [
        {
          itemLabel: '消息状态',
          itemProp: 'receiveTime',
          itemType: 'newRadio',
          searchType: 'basic',
          labelWidth: '100px',
          option: [
            // {name: '全部消息', value: '0'},
            {name: '未读', value: '1'},
            {name: '已读', value: '2'}
          ]
        },
        {
          itemLabel: '标题',
          itemProp: 'subject',
          itemType: 'input',
          labelWidth: '90px',
          searchType: 'advance'
        }
      ],
      rowInfo: {}
    }
  },
  computed: {
    // 加工为后端需要的请求格式
    computedQuery() {
      /*
       * 定义 escapeOpt 字段对应字典名 对象
       *  为重新加工结构赋值的新请求配置参数
       * */
      const querys = [
        {operation: 'EQUAL', property: 'receiveTime', relation: 'AND', group: 'filterTime'},
        {operation: 'LIKE', property: 'subject', relation: 'OR', group: 'quick'}
      ]
      const {pageSize, page, ...item} = this.queryParams
      return {
        pageBean: {pageSize, page},
        querys: querys.flatMap(i => {
          const value = item[i.property]
          if (Array.isArray(value) && !value.length) return []
          if (value === '') return []
          return {group: 'main', ...i, value}
        }),
        sorter: this.sorter.filter(i => i.direction !== '')
      }
    }
  },
  created() {
    this.loadData()
  },
  mounted() {
    this.calcTableHeight()
    this.$root.$on('resize', () => {
      this.calcTableHeight(500)
    })
  },
  methods: {
    //关闭组件
    closeComponent() {
      this.componentName = 'table'
      this.loadData()
    },

    //获取数据
    async loadData() {
      this.tableLoading = true
      const {data: {rows, total}} = await messageReceiver(this.computedQuery)
      this.tableData = rows
      this.queryParams.total = total
      this.tableLoading = false
    },

    //详情
    detailBtn(row) {
      this.componentName = 'detailDialog'

      this.$refs.detailDialog.detail(row)
    },

    // 删除
    handleDelete(row) {
      this.$confirm('确定删除选中的消息吗？', '提示', {
        confirmButtonText: '确认',
        cancelButtonText: '取消',
        type: 'warning'
      }).then(async () => {
        let ids = (row ? row.id : this.currentDataDel.join()) || ''
        if (!ids) return
        try {
          const {data: {state, message}} = await removeMessage(ids)
          if (state) {
            this.queryParams.page = 1
            await this.loadData()
            this.$message.success(message)
          } else {
            this.$message.error(message)
          }
        } catch {
          this.$message.error('删除消息失败')
        }
      }).catch(_ => _)
    },

    // 标记为已读
    markAsRead(row) {
      this.$confirm('确定标记选中的消息为已读状态吗？', '提示', {
        confirmButtonText: '确认',
        cancelButtonText: '取消',
        type: 'warning'
      }).then(async () => {
        let ids = (row ? row.id : this.currentDataDel.join()) || ''
        if (!ids) return
        try {
          const {data: {state, message}} = await markMessage(ids)
          if (state) {
            await this.loadData()
            this.$message.success(message)
          } else {
            this.$message.error(message)
          }
        } catch {
          this.$message.error('标记失败')
        }
      }).catch(_ => _)
    },

    // 顶部搜索
    handleSearch() {
      this.loadData()
    },

    //表格选中
    handleSelectionChange(val) {
      this.currentDataDel = val.flatMap(item => item.id ? item.id : [])
    },

    //分页页数
    handleCurrentChange(page) {
      this.queryParams.page = page
      this.loadData()
    },

    //分页大小
    handleSizeChange(pageSize) {
      this.queryParams.pageSize = pageSize
      this.loadData()
    },

    // 排序切换
    sortChange({prop: property, order}) {
      let direction = ''
      switch (order) {
        case 'ascending':
          direction = 'ASC'
          break
        case 'descending':
          direction = 'DESC'
          break
      }
      this.sorter.forEach(i => {
        if (i.property !== property) return i.direction = ''
        i.direction = direction
      })
      this.loadData()
    },

    // 展示弹层
    showDialog(dialog, rowInfo = {}) {
      this[dialog] = true
      this.rowInfo = rowInfo // 保存点击行信息
    },

    //  关闭详情弹层
    closeDetail() {
      if (!this.rowInfo.receiveTime) {
        this.queryParams.page = 1
        this.loadData()
      }
    }
  },
  beforeDestroy() {
  }
}
</script>

<style lang="scss" scoped>
.search-box {
  //height: 350px;
  padding: 20px 30px;
  transition: all 1s;
  background-color: #ffffff;
  border-radius: 8px;
}

.table-content {
  width: 100%;
  height: calc(100% - 110px);

  .table-list {
    width: 100%;
    height: 100%;
    cursor: default;
  }
}

/deep/ .el-table .el-table__header th {
  padding: 12px 0 !important;
  background-color: #f1f5f8 !important;
}

/deep/ .el-table .el-table__header th .cell {
  font-size: 16px;
  color: #545454;
  font-weight: 700;
  font-family: PingFang SC-Medium, PingFang SC;
  white-space: nowrap;
}

.common-tree-container {
  height: calc(100% - 70px);
  margin-top: 0.52083vw;
  padding: 1.04167vw 1.04167vw;
  background-color: #fff;
  position: relative;
  box-sizing: border-box;
  overflow-y: auto;
  border-radius: 20px;
  box-shadow: 0px 0px 0.625vw 0 rgba(92, 102, 83, 0.2);
}

.container-head {
  display: flex;
  align-items: center;

  i {
    font-size: 16px;
    margin-left: 5px;
    font-weight: 600;
    cursor: pointer;
  }
}

.move-tree /deep/ .el-tree {
  height: 90%;
  overflow-y: auto;
  border: none;
}

.cloumn-btn {
  border: 1px solid #2761ff;
  padding: 3px 10px;
  border-radius: 5px;

  &:nth-child(1) {
    margin-right: 5px;
  }

  &:nth-child(2) {
    margin-right: 5px;
  }

  &:nth-child(1):hover {
    background-color: #2761ff;
    color: #fff;
  }

  &:nth-child(2):hover {
    background-color: #2761ff;
    color: #fff;
  }

  // &:nth-child(2):hover {
  //   background-color: #f56c6c;
  //   color: #fff;
  //   border-color: #f56c6c;
  // }
  /deep/ i,
  span {
    font-size: 16px;
  }
}

/deep/ .el-tree-node__content {
  padding: 5px 0;
}
</style>
